@@ -9,6 +9,7 @@ import (
 
 	"github.com/k1LoW/tbls/dict"
 	"github.com/pkg/errors"
+	"github.com/samber/lo"
 )
 
 const (
@@ -43,25 +44,31 @@ func (labels Labels) Merge(name string) Labels {
 }
 
 func (labels Labels) Contains(name string) bool {
-	for _, l := range labels {
-		if l.Name == name {
-			return true
-		}
-	}
-	return false
+	return lo.ContainsBy(labels, func(item *Label) bool {
+		return item.Name == name
+	})
 }
 
 // Viewpoint is the struct for viewpoint information
 type Viewpoint struct {
-	Name     string   `json:"name,omitempty"`
-	Desc     string   `json:"desc,omitempty"`
-	Labels   []string `json:"labels,omitempty"`
-	Tables   []string `json:"tables,omitempty"`
-	Distance int      `json:"distance,omitempty"`
+	Name     string            `json:"name,omitempty"`
+	Desc     string            `json:"desc,omitempty"`
+	Labels   []string          `json:"labels,omitempty"`
+	Tables   []string          `json:"tables,omitempty"`
+	Distance int               `json:"distance,omitempty"`
+	Groups   []*ViewpointGroup `json:"groups,omitempty"`
 
 	Schema *Schema `json:"-"`
 }
 
+type ViewpointGroup struct {
+	Name   string   `json:"name,omitempty"`
+	Desc   string   `json:"desc,omitempty"`
+	Labels []string `json:"labels,omitempty"`
+	Tables []string `json:"tables,omitempty"`
+	Color  string   `json:"color,omitempty"`
+}
+
 type Viewpoints []*Viewpoint
 
 func (vs Viewpoints) Merge(in *Viewpoint) Viewpoints {
@@ -186,7 +193,7 @@ type Schema struct {
 }
 
 func (s *Schema) NormalizeTableName(name string) string {
-	if s.Driver != nil && (s.Driver.Name == "postgres" || s.Driver.Name == "redshift") && !strings.Contains(name, ".") {
+	if s.Driver != nil && s.Driver.Meta != nil && s.Driver.Meta.CurrentSchema != "" && (s.Driver.Name == "postgres" || s.Driver.Name == "redshift") && !strings.Contains(name, ".") {
 		return fmt.Sprintf("%s.%s", s.Driver.Meta.CurrentSchema, name)
 	}
 	return name
@@ -298,7 +305,7 @@ func (s *Schema) Repair() error {
 	}
 	// viewpoints should be created using as complete a schema as possible
 	for _, v := range s.Viewpoints {
-		cs, err := s.cloneWithoutViewpoints()
+		cs, err := s.CloneWithoutViewpoints()
 		if err != nil {
 			return errors.Wrap(err, "failed to repair viewpoint")
 		}
@@ -332,7 +339,7 @@ func (s *Schema) Clone() (c *Schema, err error) {
 	return c, nil
 }
 
-func (s *Schema) cloneWithoutViewpoints() (c *Schema, err error) {
+func (s *Schema) CloneWithoutViewpoints() (c *Schema, err error) {
 	defer func() {
 		err = errors.WithStack(err)
 	}()
@@ -507,8 +514,8 @@ func (t *Table) hasColumnWithValues(name string) bool {
 }
 
 func (t *Table) ShowColumn(name string, hideColumns []string) bool {
-	hideColumns = unique(append(DefaultHideColumns, hideColumns...))
-	if contains(hideColumns, name) {
+	hideColumns = lo.Uniq(append(DefaultHideColumns, hideColumns...))
+	if lo.Contains(hideColumns, name) {
 		return t.hasColumnWithValues(name)
 	}
 	return true
@@ -571,47 +578,9 @@ func (t *Table) CollectTablesAndRelations(distance int, root bool) ([]*Table, []
 	return uTables, uRelations, nil
 }
 
-func (t *Table) Contains(ts []*Table) bool {
-	for _, tt := range ts {
-		if t.Name == tt.Name {
-			return true
-		}
-	}
-	return false
-}
-
-func unique(in []string) []string {
-	u := []string{}
-	m := map[string]struct{}{}
-	for _, s := range in {
-		if _, ok := m[s]; ok {
-			continue
-		}
-		u = append(u, s)
-		m[s] = struct{}{}
-	}
-	return u
-}
-
-func contains(s []string, e string) bool {
-	for _, v := range s {
-		if e == v {
-			return true
-		}
-	}
-	return false
-}
-
 func sameElements(a, b []string) bool {
-	for _, aa := range a {
-		if !contains(b, aa) {
-			return false
-		}
+	if len(a) == len(b) && lo.Every(a, b) {
+		return true
 	}
-	for _, bb := range b {
-		if !contains(a, bb) {
-			return false
-		}
-	}
-	return true
+	return false
 }
